// SPDX-License-Identifier: ISC

function maybeToArray(value) {
	return Array.isArray(value) ? value : [value];
}

function esac(caseValue) {
	const result = {
		case: _ => ({
			then: _ => result,
			thenDo: _ => result,
			thenDoFallthrough: _ => result,
		}),
		caseLazy: _ => ({
			then: _ => result,
			thenDo: _ => result,
			thenDoFallthrough: _ => result,
		}),
		default: _ => caseValue,
		defaultDo: _ => caseValue,
	};

	return result;
}

function fallthrough() {
	const result = {
		case: _ => ({
			then: thenValue => esac(thenValue),
			thenDo: thenFn => esac(thenFn()),
			thenDoFallthrough: thenFn => { thenFn(); return result; },
		}),
		caseLazy: _ => ({
			then: thenValue => esac(thenValue),
			thenDo: thenFn => esac(thenFn()),
			thenDoFallthrough: thenFn => { thenFn(); return result; },
		}),
		default: defaultValue => defaultValue,
		defaultDo: defaultFn => defaultFn(),
	};

	return result;
}

function hctiws(query) {
	const result = {
		case: (...values) => ({
			then: thenValue => values.includes(query) ? esac(thenValue) : result,
			thenDo: thenFn => values.includes(query) ? esac(thenFn()) : result,
			thenDoFallthrough: thenFn => values.includes(query) ? fallthrough(thenFn()) : result,
		}),
		caseLazy: caseFn => maybeToArray(caseFn()).includes(query) ? {
			then: thenValue => esac(thenValue),
			thenDo: thenFn => esac(thenFn()),
			thenDoFallthrough: thenFn => fallthrough(thenFn()),
		} : {
			then: _ => result,
			thenDo: _ => result,
			thenDoFallthrough: _ => result,
		},
		default: defaultValue => defaultValue,
		defaultDo: defaultFn => defaultFn(),
	};

	return result;
}

export default hctiws;
